@@ -5,21 +5,29 @@
 This module contains the Python API for interacting with the Ledger API.
 """
 from ._logging import LOG
-
-from .client import run, simple_client, async_network, Network, SimplePartyClient, AIOPartyClient
-from .model.core import ContractId, ContractData, DazlError, Party
-from .model.writing import create, exercise, exercise_by_key, create_and_exercise, \
-    Command, CreateCommand, ExerciseCommand, ExerciseByKeyCommand, CreateAndExerciseCommand
-from .util.logging import setup_default_logger
-from .prim import FrozenDict as frozendict
+from .client import AIOPartyClient, Network, SimplePartyClient, async_network, run, simple_client
+from .model.core import ContractData, ContractId, DazlError, Party
+from .model.writing import (
+    Command,
+    CreateAndExerciseCommand,
+    CreateCommand,
+    ExerciseByKeyCommand,
+    ExerciseCommand,
+    create,
+    create_and_exercise,
+    exercise,
+    exercise_by_key,
+)
 from .pretty.table import write_acs
-
+from .prim import FrozenDict as frozendict
+from .util.logging import setup_default_logger
 
 try:
     # This method is undocumented, but is required to read large size of model files when using
     # the C++ implementation.
     # noinspection PyPackageRequirements,PyUnresolvedReferences,PyProtectedMember
     from google.protobuf.pyext import _message
+
     _message.SetAllowOversizeProtos(True)
 
 except ImportError:
@@ -38,25 +46,27 @@ def _get_version() -> str:
      2. Use the value from the local pyproject.toml file (this is used when
         running dazl from source).
     """
-    import pkg_resources
     from ast import literal_eval
     from configparser import ConfigParser
     from pathlib import Path
+
+    import pkg_resources
+
     try:
-        return pkg_resources.require('dazl')[0].version
+        return pkg_resources.require("dazl")[0].version
     except pkg_resources.DistributionNotFound:
         pass
     except Exception:
-        return 'unknown'
+        return "unknown"
 
     try:
         config = ConfigParser()
-        config.read(Path(__file__).parent.parent / 'pyproject.toml')
-        if 'tool.poetry' in config:
-            poetry_section = config['tool.poetry']
-            return literal_eval(poetry_section['version'])
+        config.read(Path(__file__).parent.parent / "pyproject.toml")
+        if "tool.poetry" in config:
+            poetry_section = config["tool.poetry"]
+            return literal_eval(poetry_section["version"])
     except Exception:  # noqa
-        return 'unknown'
+        return "unknown"
 
 
 __version__ = _get_version()
