@@ -5,22 +5,33 @@
 This module contains configuration parameters used by the rest of the library.
 """
 
-import warnings
 import argparse
-
-from dataclasses import dataclass, field, fields, asdict
-from typing import Any, Collection, List, Mapping, Optional, Sequence, Type, TypeVar, \
-    TYPE_CHECKING
+from dataclasses import asdict, dataclass, field, fields
+from typing import TYPE_CHECKING, Any, Collection, List, Mapping, Optional, Sequence, Type, TypeVar
+import warnings
 
 from .. import LOG
 from ..model.core import ConfigurationError, Party
-from ..util.config_meta import config_field, \
-    BOOLEAN_TYPE, COUNT_TYPE, LOG_LEVEL_TYPE, PARTIES_TYPE, PATH_TYPE, PACKAGE_IDS_TYPE, \
-    PORT_TYPE, SECONDS_TYPE, STRING_TYPE, URL_TYPE, VERIFY_SSL_TYPE, config_fields, add_argument
+from ..util.config_meta import (
+    BOOLEAN_TYPE,
+    COUNT_TYPE,
+    LOG_LEVEL_TYPE,
+    PACKAGE_IDS_TYPE,
+    PARTIES_TYPE,
+    PATH_TYPE,
+    PORT_TYPE,
+    SECONDS_TYPE,
+    STRING_TYPE,
+    URL_TYPE,
+    VERIFY_SSL_TYPE,
+    add_argument,
+    config_field,
+    config_fields,
+)
 
 # If this environment variable is set, is used in place of a configuration file if none is supplied
 # on the command-line.
-DAZL_CONFIG_ENV = 'DAZL_CONFIG'
+DAZL_CONFIG_ENV = "DAZL_CONFIG"
 
 
 # Define a type alias for things that quack like an ArgumentParser
@@ -39,64 +50,63 @@ class URLConfig:
     """
 
     url: str = config_field(
-        'URL where the Ledger API is hosted',
+        "URL where the Ledger API is hosted",
         param_type=URL_TYPE,
-        short_alias='u',
-        environment_variable='DAML_LEDGER_URL',
-        deprecated_alias='participant_url')
+        short_alias="u",
+        environment_variable="DAML_LEDGER_URL",
+        deprecated_alias="participant_url",
+    )
 
     ca_file: Optional[str] = config_field(
-        'server certificate authority file',
-        param_type=PATH_TYPE,
-        default_value=None)
+        "server certificate authority file", param_type=PATH_TYPE, default_value=None
+    )
 
-    cert_file: Optional[str] = config_field(
-        'client certificate file',
-        param_type=PATH_TYPE)
+    cert_file: Optional[str] = config_field("client certificate file", param_type=PATH_TYPE)
 
     cert_key_file: Optional[str] = config_field(
-        'client certificate and key file',
-        param_type=PATH_TYPE)
+        "client certificate and key file", param_type=PATH_TYPE
+    )
 
     verify_ssl: Optional[str] = config_field(
-        'level of verification to use for SSL',
-        param_type=VERIFY_SSL_TYPE)
+        "level of verification to use for SSL", param_type=VERIFY_SSL_TYPE
+    )
 
     poll_interval: Optional[float] = config_field(
-        'polling interval for receiving new events',
-        param_type=SECONDS_TYPE,
-        default_value=0.1)
+        "polling interval for receiving new events", param_type=SECONDS_TYPE, default_value=0.1
+    )
 
     connect_timeout: Optional[float] = config_field(
-        'number of seconds before giving up on a connection',
+        "number of seconds before giving up on a connection",
         param_type=SECONDS_TYPE,
-        default_value=DEFAULT_CONNECT_TIMEOUT_SECONDS)
+        default_value=DEFAULT_CONNECT_TIMEOUT_SECONDS,
+    )
 
     eager_package_fetch: Optional[bool] = config_field(
-        'whether to fetch all packages on startup',
-        param_type=BOOLEAN_TYPE,
-        default_value=True)
+        "whether to fetch all packages on startup", param_type=BOOLEAN_TYPE, default_value=True
+    )
 
     enable_http_proxy: Optional[bool] = config_field(
-        'whether to allow the use of HTTP proxies',
-        param_type=BOOLEAN_TYPE,
-        default_value=True)
+        "whether to allow the use of HTTP proxies", param_type=BOOLEAN_TYPE, default_value=True
+    )
 
     package_lookup_timeout: Optional[float] = config_field(
-        'number of seconds before giving up on a package',
+        "number of seconds before giving up on a package",
         param_type=SECONDS_TYPE,
-        default_value=DEFAULT_CONNECT_TIMEOUT_SECONDS)
+        default_value=DEFAULT_CONNECT_TIMEOUT_SECONDS,
+    )
 
     application_name: Optional[str] = config_field(
-        'The name that this application uses to identify itself to the ledger.',
+        "The name that this application uses to identify itself to the ledger.",
         param_type=STRING_TYPE,
-        default_value='DAZL-Client')
+        default_value="DAZL-Client",
+    )
 
     log_level: Optional[int] = config_field(
-        'logging level for events for this party',
+        "logging level for events for this party",
         param_type=LOG_LEVEL_TYPE,
-        environment_variable='DAZL_LOG_LEVEL',
-        short_alias='l')
+        environment_variable="DAZL_LOG_LEVEL",
+        short_alias="l",
+    )
 
 
 @dataclass(frozen=True)
@@ -106,23 +116,26 @@ class _PartyConfig(URLConfig):
     """
 
     max_event_block_size: Optional[int] = config_field(
-        'Maximum number of blocks to read in a single call.',
+        "Maximum number of blocks to read in a single call.",
         param_type=COUNT_TYPE,
-        default_value=100)
+        default_value=100,
+    )
 
     max_command_batch_size: Optional[int] = config_field(
-        'Maximum number of commands to batch up in a single transaction',
+        "Maximum number of commands to batch up in a single transaction",
         param_type=COUNT_TYPE,
-        default_value=100)
+        default_value=100,
+    )
 
     max_command_batch_timeout: Optional[float] = config_field(
-        'Maximum number of seconds to wait before sending out a command',
+        "Maximum number of seconds to wait before sending out a command",
         param_type=SECONDS_TYPE,
-        default_value=0.25)
+        default_value=0.25,
+    )
 
     party_groups: Optional[Collection[str]] = config_field(
-        'comma-separated list of broadcast parties',
-        param_type=PARTIES_TYPE)
+        "comma-separated list of broadcast parties", param_type=PARTIES_TYPE
+    )
 
 
 @dataclass(frozen=True)
@@ -136,102 +149,101 @@ class _NetworkConfig(URLConfig):
     """
 
     log_level: Optional[int] = config_field(
-        'logging level for events for this party',
+        "logging level for events for this party",
         param_type=LOG_LEVEL_TYPE,
-        environment_variable='DAZL_LOG_LEVEL',
-        short_alias='l')
+        environment_variable="DAZL_LOG_LEVEL",
+        short_alias="l",
+    )
 
     max_connection_count: Optional[int] = config_field(
-        'Number of concurrent HTTP connections to have outstanding.',
+        "Number of concurrent HTTP connections to have outstanding.",
         param_type=COUNT_TYPE,
-        default_value=20)
+        default_value=20,
+    )
 
     quiet_timeout: Optional[float] = config_field(
         'Number of seconds to wait after the client "thinks" it\'s done to hang around for',
         param_type=SECONDS_TYPE,
-        default_value=1)
+        default_value=1,
+    )
 
     use_acs_service: bool = config_field(
-        'Use Active Contract Set service instead of reading from the Transaction event stream',
+        "Use Active Contract Set service instead of reading from the Transaction event stream",
         param_type=BOOLEAN_TYPE,
-        default_value=True)
+        default_value=True,
+    )
 
     idle_timeout: Optional[float] = config_field(
-        'Maximum number of seconds of idle activity before automatically closing the client',
+        "Maximum number of seconds of idle activity before automatically closing the client",
         param_type=SECONDS_TYPE,
-        default_value=120)
+        default_value=120,
+    )
 
     max_consequence_depth: Optional[int] = config_field(
-        'The maximum number of times to wait for all parties to arrive at the same offset' +
-        'before failing with an error',
+        "The maximum number of times to wait for all parties to arrive at the same offset"
+        + "before failing with an error",
         param_type=COUNT_TYPE,
-        default_value=50)
+        default_value=50,
+    )
 
     party_groups: Optional[Collection[str]] = config_field(
-        'comma-separated list of broadcast parties',
-        param_type=PARTIES_TYPE)
+        "comma-separated list of broadcast parties", param_type=PARTIES_TYPE
+    )
 
     package_ids: Optional[Collection[str]] = config_field(
-        'comma-separated list of package IDs to listen on',
-        param_type=PACKAGE_IDS_TYPE)
+        "comma-separated list of package IDs to listen on", param_type=PACKAGE_IDS_TYPE
+    )
 
     server_host: Optional[str] = config_field(
-        'Server listening host. Used for OAuth web application flow callbacks.',
+        "Server listening host. Used for OAuth web application flow callbacks.",
         param_type=STRING_TYPE,
-        environment_variable='DAZL_SERVER_HOST')
+        environment_variable="DAZL_SERVER_HOST",
+    )
 
     server_port: Optional[int] = config_field(
-        'Server listening port. Used for OAuth web application flow callbacks.',
+        "Server listening port. Used for OAuth web application flow callbacks.",
         param_type=PORT_TYPE,
-        environment_variable='DAZL_SERVER_PORT')
+        environment_variable="DAZL_SERVER_PORT",
+    )
 
-    oauth_client_id: Optional[str] = config_field(
-        'OAuth client ID',
-        param_type=STRING_TYPE)
+    oauth_client_id: Optional[str] = config_field("OAuth client ID", param_type=STRING_TYPE)
 
     oauth_client_secret: Optional[str] = config_field(
-        'OAuth client secret (implies web application flow or backend application flow)',
-        param_type=STRING_TYPE)
+        "OAuth client secret (implies web application flow or backend application flow)",
+        param_type=STRING_TYPE,
+    )
 
-    oauth_token: Optional[str] = config_field(
-        'OAuth token',
-        param_type=STRING_TYPE)
+    oauth_token: Optional[str] = config_field("OAuth token", param_type=STRING_TYPE)
 
-    oauth_refresh_token: Optional[str] = config_field(
-        'OAuth refresh token',
-        param_type=STRING_TYPE)
+    oauth_refresh_token: Optional[str] = config_field("OAuth refresh token", param_type=STRING_TYPE)
 
     oauth_id_token: Optional[str] = config_field(
-        'OpenID token (JWT formatted)',
-        param_type=STRING_TYPE)
+        "OpenID token (JWT formatted)", param_type=STRING_TYPE
+    )
 
-    oauth_token_uri: Optional[str] = config_field(
-        'OAuth token URL',
-        param_type=STRING_TYPE)
+    oauth_token_uri: Optional[str] = config_field("OAuth token URL", param_type=STRING_TYPE)
 
     oauth_redirect_uri: Optional[str] = config_field(
-        'OAuth redirect URI (implies web application flow)',
-        param_type=STRING_TYPE)
+        "OAuth redirect URI (implies web application flow)", param_type=STRING_TYPE
+    )
 
     oauth_auth_url: Optional[str] = config_field(
-        'OAuth auth URL (implies mobile application flow)',
-        param_type=STRING_TYPE)
+        "OAuth auth URL (implies mobile application flow)", param_type=STRING_TYPE
+    )
 
-    oauth_ca_file: Optional[str] = config_field(
-        'OAuth CA CA Bundle File',
-        param_type=STRING_TYPE)
+    oauth_ca_file: Optional[str] = config_field("OAuth CA CA Bundle File", param_type=STRING_TYPE)
 
     oauth_audience: Optional[str] = config_field(
-        'OAuth audience (Implies Client Credentials Flow)',
-        param_type=STRING_TYPE)
+        "OAuth audience (Implies Client Credentials Flow)", param_type=STRING_TYPE
+    )
 
     oauth_legacy_username: Optional[str] = config_field(
-        'OAuth username (implies legacy application flow)',
-        param_type=STRING_TYPE)
+        "OAuth username (implies legacy application flow)", param_type=STRING_TYPE
+    )
 
     oauth_legacy_password: Optional[str] = config_field(
-        'OAuth password (implies legacy application flow)',
-        param_type=STRING_TYPE)
+        "OAuth password (implies legacy application flow)", param_type=STRING_TYPE
+    )
 
 
 @dataclass(frozen=True)
@@ -243,13 +255,15 @@ class _FlatConfig(_NetworkConfig, _PartyConfig):
     This configuration object is used as an intermediate holding object when parsing parameters
     from environment variables or command-line arguments.
     """
+
     parties: Collection[Party] = config_field(
-        'comma-separated list of parties serviced by a participant node',
+        "comma-separated list of parties serviced by a participant node",
         param_type=PARTIES_TYPE,
-        long_alias='party',
-        short_alias='p',
-        environment_variable='DAML_LEDGER_PARTY',
-        default_value=())
+        long_alias="party",
+        short_alias="p",
+        environment_variable="DAML_LEDGER_PARTY",
+        default_value=(),
+    )
 
 
 @dataclass(frozen=True)
@@ -257,11 +271,13 @@ class PartyConfig(_PartyConfig, _NetworkConfig):
     party: Optional[Party] = None
 
 
-C = TypeVar('C', bound='_TopLevelConfig')
+C = TypeVar("C", bound="_TopLevelConfig")
+
+
 @dataclass(frozen=True)
 class _TopLevelConfig:
     @classmethod
-    def parse_kwargs(cls: Type[C], *config: 'C', **kwargs) -> 'C':
+    def parse_kwargs(cls: Type[C], *config: "C", **kwargs) -> "C":
         """
         Create a :class:`NetworkConfig` for the configuration settings.
 
@@ -283,7 +299,7 @@ class _TopLevelConfig:
         return configurations[0].validate()
 
     @classmethod
-    def get_config(cls: Type[C], args: 'argparse.Namespace') -> 'C':
+    def get_config(cls: Type[C], args: "argparse.Namespace") -> "C":
         """
         Convert an ``argparse.Namespace`` to a fully-formed and valid :class:`NetworkConfig`.
         """
@@ -294,10 +310,10 @@ class _TopLevelConfig:
         return network_config.validate()
 
     @classmethod
-    def unflatten(cls: Type[C], config: '_FlatConfig') -> 'C':
+    def unflatten(cls: Type[C], config: "_FlatConfig") -> "C":
         raise NotImplementedError
 
-    def validate(self: C) -> 'C':
+    def validate(self: C) -> "C":
         """
         Return this config, or raise a ConfigurationError if there is something wrong with the
         parameters of this object.
@@ -310,10 +326,10 @@ class _TopLevelConfig:
 
 @dataclass(frozen=True)
 class NetworkConfig(_NetworkConfig, _TopLevelConfig):
-    parties: 'Sequence[PartyConfig]' = field(default_factory=tuple)
+    parties: "Sequence[PartyConfig]" = field(default_factory=tuple)
 
     @classmethod
-    def unflatten(cls, config: '_FlatConfig') -> 'NetworkConfig':
+    def unflatten(cls, config: "_FlatConfig") -> "NetworkConfig":
         """
         Convert a :class:`_FlatConfig` to a :class:`NetworkConfig`.
         """
@@ -323,13 +339,13 @@ class NetworkConfig(_NetworkConfig, _TopLevelConfig):
         parties = tuple(PartyConfig(party=party, **party_dict) for party in config.parties)
         return NetworkConfig(parties=parties, **network_dict)
 
-    def validate(self) -> 'NetworkConfig':
+    def validate(self) -> "NetworkConfig":
         failures = set()
-        LOG.debug('Configuration: %s', self)
+        LOG.debug("Configuration: %s", self)
         if self.parties:
             for party_config in self.parties:
                 if not party_config.url:
-                    failures.add(f'Party {party_config.party} has no Ledger API URL')
+                    failures.add(f"Party {party_config.party} has no Ledger API URL")
 
         if failures:
             raise ConfigurationError(failures)
@@ -345,19 +361,21 @@ class AnonymousNetworkConfig(_NetworkConfig, _TopLevelConfig):
     """
 
     @classmethod
-    def unflatten(cls, config: '_FlatConfig') -> 'AnonymousNetworkConfig':
+    def unflatten(cls, config: "_FlatConfig") -> "AnonymousNetworkConfig":
         """
         Convert a :class:`_FlatConfig` to a :class:`NetworkConfig`.
         """
         config_dict = {k: v for k, v in asdict(config).items() if v is not None}
-        network_dict = {fld.name: config_dict.get(fld.name) for fld in fields(AnonymousNetworkConfig)}
+        network_dict = {
+            fld.name: config_dict.get(fld.name) for fld in fields(AnonymousNetworkConfig)
+        }
         return AnonymousNetworkConfig(**network_dict)
 
-    def validate(self) -> 'AnonymousNetworkConfig':
+    def validate(self) -> "AnonymousNetworkConfig":
         failures = []
 
         if not self.url:
-            failures.append('URL must be specified')
+            failures.append("URL must be specified")
 
         if failures:
             raise ConfigurationError(failures)
@@ -366,9 +384,8 @@ class AnonymousNetworkConfig(_NetworkConfig, _TopLevelConfig):
 
 
 def configure_parser(
-        arg_parser: 'ArgumentParser',
-        config_file_support: bool = False,
-        parties: bool = True):
+    arg_parser: "ArgumentParser", config_file_support: bool = False, parties: bool = True
+):
     """
     Add standard options to an arg parser (later to be extracted out by ``get_config``).
 
@@ -381,19 +398,20 @@ def configure_parser(
     """
     if config_file_support:
         from os import environ
+
         default_config = environ.get(DAZL_CONFIG_ENV)
-        group = arg_parser.add_argument_group('Configuration Settings')
-        group.add_argument('--config', help='path to a YAML config file', default=default_config)
+        group = arg_parser.add_argument_group("Configuration Settings")
+        group.add_argument("--config", help="path to a YAML config file", default=default_config)
 
     if parties:
         party_config_fields = [fld.name for fld, _ in config_fields(_PartyConfig)]
 
-        group = arg_parser.add_argument_group('Overall Settings')
+        group = arg_parser.add_argument_group("Overall Settings")
         for fld, param in config_fields(_FlatConfig):
             if fld.name not in party_config_fields:
                 add_argument(group, fld.name, param)
 
-        group = arg_parser.add_argument_group('Per-Party Settings')
+        group = arg_parser.add_argument_group("Per-Party Settings")
         for fld, param in config_fields(_PartyConfig):
             add_argument(group, fld.name, param)
 
@@ -404,7 +422,7 @@ def configure_parser(
     return arg_parser
 
 
-def fetch_config(path_or_url: 'Optional[str]') -> 'Optional[str]':
+def fetch_config(path_or_url: "Optional[str]") -> "Optional[str]":
     """
     Attempts to fetch a config file from what looks like either a path or a URL.
 
@@ -414,38 +432,39 @@ def fetch_config(path_or_url: 'Optional[str]') -> 'Optional[str]':
     if not path_or_url:
         return None
 
-    from urllib.error import URLError, HTTPError
+    from urllib.error import HTTPError, URLError
     from urllib.parse import urlparse
     from urllib.request import urlopen
 
     u = urlparse(path_or_url)
-    if not u.scheme or u.scheme == 'file':
+    if not u.scheme or u.scheme == "file":
         try:
-            with open(u.path, 'r') as buf:
+            with open(u.path, "r") as buf:
                 config_str = buf.read()
         except FileNotFoundError:
-            raise ConfigurationError([f'Config file not found: {u.path}'])
+            raise ConfigurationError([f"Config file not found: {u.path}"])
     else:
         try:
             with urlopen(path_or_url) as buf:
-                config_str = buf.read().decode('utf8')
+                config_str = buf.read().decode("utf8")
         except HTTPError as error:
-            raise ConfigurationError([f'HTTP {error.code}: {path_or_url}'])
+            raise ConfigurationError([f"HTTP {error.code}: {path_or_url}"])
         except URLError as error:
             if isinstance(error.args[0], ConnectionRefusedError):
-                raise ConfigurationError([f'HTTP unreachable: {path_or_url}'])
+                raise ConfigurationError([f"HTTP unreachable: {path_or_url}"])
             else:
                 raise ConfigurationError([error.args[0]])
 
     return config_str
 
 
-def _get_env_defaults() -> 'Mapping[str, Any]':
+def _get_env_defaults() -> "Mapping[str, Any]":
     """
     Produce a mapping of configuration keys to values set in the environment through environment
     variables.
     """
     from os import getenv
+
     kwargs = {}
     for fld, param in config_fields(_FlatConfig):
         if param.environment_variable:
@@ -455,17 +474,18 @@ def _get_env_defaults() -> 'Mapping[str, Any]':
     return kwargs
 
 
-def _parse_args_ns(args: 'argparse.Namespace') -> 'Mapping[str, Any]':
+def _parse_args_ns(args: "argparse.Namespace") -> "Mapping[str, Any]":
     """
     Convert the values
     :param args:
     :return:
     """
-    return {fld.name: getattr(args, fld.name)
-            for fld in fields(_FlatConfig) if hasattr(args, fld.name)}
+    return {
+        fld.name: getattr(args, fld.name) for fld in fields(_FlatConfig) if hasattr(args, fld.name)
+    }
 
 
-def _parse_args_dict(d: 'Mapping[str, Any]') -> 'Mapping[str, Any]':
+def _parse_args_dict(d: "Mapping[str, Any]") -> "Mapping[str, Any]":
     """
     Convert the key-value pairs in this mapping to keys that are defined on :class:`_FlatConfig`.
 
@@ -485,8 +505,11 @@ def _parse_args_dict(d: 'Mapping[str, Any]') -> 'Mapping[str, Any]':
         if value is None:
             for key in param.deprecated_aliases:
                 if d.get(key) is not None:
-                    warnings.warn(f'The {key} option is deprecated. Please use {fld.name} instead.',
-                                  DeprecationWarning, stacklevel=2)
+                    warnings.warn(
+                        f"The {key} option is deprecated. Please use {fld.name} instead.",
+                        DeprecationWarning,
+                        stacklevel=2,
+                    )
                     value = d.get(key)
                     break
 
@@ -495,10 +518,10 @@ def _parse_args_dict(d: 'Mapping[str, Any]') -> 'Mapping[str, Any]':
                 try:
                     value = param.param_type.value_ctor(value)
                 except:
-                    LOG.exception('Config key: %r; could not parse value: %r', fld.name, value)
+                    LOG.exception("Config key: %r; could not parse value: %r", fld.name, value)
 
             kwargs[fld.name] = value
 
     if unknown_fields:
-        raise ValueError(f'unknown kwargs: {sorted(unknown_fields)}')
+        raise ValueError(f"unknown kwargs: {sorted(unknown_fields)}")
     return kwargs
