@@ -17,12 +17,11 @@ Ledger API.
 .. autoclass:: ExerciseCommand
    :members:
 """
+from dataclasses import dataclass, fields
+from datetime import timedelta
+from typing import TYPE_CHECKING, Any, Collection, List, Mapping, Optional, Sequence, Union
 import uuid
 import warnings
-from datetime import timedelta
-from typing import Any, Collection, List, Mapping, Optional, Sequence, Union, TYPE_CHECKING
-
-from dataclasses import dataclass, fields
 
 from ..damlast.daml_lf_1 import TypeConName
 from ..damlast.daml_types import con
@@ -33,13 +32,13 @@ from ..util.typing import safe_cast
 
 if TYPE_CHECKING:
     with warnings.catch_warnings():
-        warnings.simplefilter('ignore', DeprecationWarning)
+        warnings.simplefilter("ignore", DeprecationWarning)
         from .types import Type, TypeReference
 
     from ..values import Context, ValueMapper
 
-CommandsOrCommandSequence = Union[None, 'Command', Sequence[Optional['Command']]]
-EventHandlerResponse = Union[CommandsOrCommandSequence, 'CommandBuilder', 'CommandPayload']
+CommandsOrCommandSequence = Union[None, "Command", Sequence[Optional["Command"]]]
+EventHandlerResponse = Union[CommandsOrCommandSequence, "CommandBuilder", "CommandPayload"]
 
 
 class Command:
@@ -62,28 +61,31 @@ class CreateCommand(Command):
 
         The arguments to the create (as a ``dict``).
     """
-    __slots__ = ('template_type', 'arguments', '_template_type_deprecated')
 
-    template_type: 'TypeConName'
-    arguments: 'Mapping[str, Any]'
-    _template_type_deprecated: 'TypeReference'
+    __slots__ = ("template_type", "arguments", "_template_type_deprecated")
 
-    def __init__(self, template: 'Union[str, TypeConName, Type]', arguments=None):
+    template_type: "TypeConName"
+    arguments: "Mapping[str, Any]"
+    _template_type_deprecated: "TypeReference"
+
+    def __init__(self, template: "Union[str, TypeConName, Type]", arguments=None):
         from ..damlast.compat import parse_template
 
         template_type, template_type_deprecated = parse_template(template)
-        object.__setattr__(self, 'template_type', template_type)
-        object.__setattr__(self, 'arguments', arguments or dict())
-        object.__setattr__(self, '_template_type_deprecated', template_type_deprecated)
+        object.__setattr__(self, "template_type", template_type)
+        object.__setattr__(self, "arguments", arguments or dict())
+        object.__setattr__(self, "_template_type_deprecated", template_type_deprecated)
 
     @property
-    def template(self) -> 'TypeReference':
+    def template(self) -> "TypeReference":
         warnings.warn(
             "CreateCommand.template is deprecated; use CreateCommand.template_type instead.",
-            DeprecationWarning, stacklevel=2)
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self._template_type_deprecated
 
-    def replace(self, template: 'Union[None, str, Type]' = None, arguments=None):
+    def replace(self, template: "Union[None, str, Type]" = None, arguments=None):
         """
         Create a new :class:`CreateCommand` with the same identifier as this command, but with new
         values for its parameters.
@@ -95,22 +97,27 @@ class CreateCommand(Command):
         """
         warnings.warn(
             "CreateCommand.replace is deprecated; simply construct a CreateCommand with the "
-            "desired values instead.", DeprecationWarning, stacklevel=2)
+            "desired values instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
 
         with warnings.catch_warnings():
-            warnings.simplefilter('ignore', DeprecationWarning)
+            warnings.simplefilter("ignore", DeprecationWarning)
 
             from .types import Type, UnresolvedTypeReference
 
             if template is not None:
-                template = template if isinstance(template, Type) \
-                    else UnresolvedTypeReference(template)
+                template = (
+                    template if isinstance(template, Type) else UnresolvedTypeReference(template)
+                )
             return CreateCommand(
                 template if template is not None else self.template_type,
-                arguments if arguments is not None else self.arguments)
+                arguments if arguments is not None else self.arguments,
+            )
 
     def __repr__(self):
-        return f'<create {self.template_type} {self.arguments}>'
+        return f"<create {self.template_type} {self.arguments}>"
 
 
 @dataclass(init=False, frozen=True)
@@ -142,16 +149,17 @@ class ExerciseCommand(Command):
 
     .. automethod:: replace
     """
-    __slots__ = ('contract', 'choice', 'arguments')
 
-    contract: 'ContractId'
+    __slots__ = ("contract", "choice", "arguments")
+
+    contract: "ContractId"
     choice: str
-    arguments: 'Optional[Any]'
+    arguments: "Optional[Any]"
 
-    def __init__(self, contract: 'ContractId', choice: str, arguments: 'Optional[Any]' = None):
-        object.__setattr__(self, 'contract', safe_cast(ContractId, contract))
-        object.__setattr__(self, 'choice', safe_cast(str, choice))
-        object.__setattr__(self, 'arguments', dict(arguments) if arguments is not None else dict())
+    def __init__(self, contract: "ContractId", choice: str, arguments: "Optional[Any]" = None):
+        object.__setattr__(self, "contract", safe_cast(ContractId, contract))
+        object.__setattr__(self, "choice", safe_cast(str, choice))
+        object.__setattr__(self, "arguments", dict(arguments) if arguments is not None else dict())
 
     def replace(self, contract=None, choice=None, arguments=None):
         """
@@ -169,11 +177,14 @@ class ExerciseCommand(Command):
         """
         warnings.warn(
             "ExerciseCommand.replace is deprecated; simply construct a ExerciseCommand with the "
-            "desired values instead.", DeprecationWarning)
+            "desired values instead.",
+            DeprecationWarning,
+        )
         return ExerciseCommand(
             contract if contract is not None else self.contract,
             choice if choice is not None else self.choice,
-            arguments if arguments is not None else self.arguments)
+            arguments if arguments is not None else self.arguments,
+        )
 
     def __repr__(self):
         return f"<exercise '{self.contract.value}' {self.choice} with {self.arguments}>"
@@ -181,63 +192,71 @@ class ExerciseCommand(Command):
 
 @dataclass(init=False, frozen=True)
 class ExerciseByKeyCommand(Command):
-    template_type: 'TypeConName'
-    contract_key: 'Any'
+    template_type: "TypeConName"
+    contract_key: "Any"
     choice: str
-    choice_argument: 'Any'
-    _template_type_deprecated: 'TypeReference'
+    choice_argument: "Any"
+    _template_type_deprecated: "TypeReference"
 
     def __init__(
-            self,
-            template: 'Union[str, TypeConName, TypeReference]',
-            contract_key: 'Any',
-            choice: str,
-            choice_argument: 'Any'):
+        self,
+        template: "Union[str, TypeConName, TypeReference]",
+        contract_key: "Any",
+        choice: str,
+        choice_argument: "Any",
+    ):
         from ..damlast.compat import parse_template
 
         template_type, template_type_deprecated = parse_template(template)
-        object.__setattr__(self, 'template_type', template_type)
-        object.__setattr__(self, 'contract_key', contract_key)
-        object.__setattr__(self, 'choice', choice)
-        object.__setattr__(self, 'choice_argument', choice_argument)
-        object.__setattr__(self, '_template_type_deprecated', template_type_deprecated)
+        object.__setattr__(self, "template_type", template_type)
+        object.__setattr__(self, "contract_key", contract_key)
+        object.__setattr__(self, "choice", choice)
+        object.__setattr__(self, "choice_argument", choice_argument)
+        object.__setattr__(self, "_template_type_deprecated", template_type_deprecated)
 
     @property
-    def template(self) -> 'TypeReference':
+    def template(self) -> "TypeReference":
         warnings.warn(
             "ExerciseByKeyCommand.template is deprecated; use ExerciseByKeyCommand.template_type "
-            "instead.", DeprecationWarning, stacklevel=2)
+            "instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self._template_type_deprecated
 
 
 @dataclass(init=False, frozen=True)
 class CreateAndExerciseCommand(Command):
-    template_type: 'TypeConName'
-    arguments: 'Mapping[str, Any]'
+    template_type: "TypeConName"
+    arguments: "Mapping[str, Any]"
     choice: str
-    choice_argument: 'Any'
-    _template_type_deprecated: 'TypeReference'
+    choice_argument: "Any"
+    _template_type_deprecated: "TypeReference"
 
     def __init__(
-            self,
-            template: 'Union[str, TypeConName, TypeReference]',
-            arguments: 'Mapping[str, Any]',
-            choice: str,
-            choice_argument: 'Any'):
+        self,
+        template: "Union[str, TypeConName, TypeReference]",
+        arguments: "Mapping[str, Any]",
+        choice: str,
+        choice_argument: "Any",
+    ):
         from ..damlast.compat import parse_template
 
         template_type, template_type_deprecated = parse_template(template)
-        object.__setattr__(self, 'template_type', template_type)
-        object.__setattr__(self, 'arguments', arguments)
-        object.__setattr__(self, 'choice', choice)
-        object.__setattr__(self, 'choice_argument', choice_argument)
-        object.__setattr__(self, '_template_type_deprecated', template_type_deprecated)
+        object.__setattr__(self, "template_type", template_type)
+        object.__setattr__(self, "arguments", arguments)
+        object.__setattr__(self, "choice", choice)
+        object.__setattr__(self, "choice_argument", choice_argument)
+        object.__setattr__(self, "_template_type_deprecated", template_type_deprecated)
 
     @property
-    def template(self) -> 'TypeReference':
+    def template(self) -> "TypeReference":
         warnings.warn(
             "CreateAndExerciseCommand.template is deprecated; use "
-            "CreateAndExerciseCommand.template_type instead.", DeprecationWarning, stacklevel=2)
+            "CreateAndExerciseCommand.template_type instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self._template_type_deprecated
 
 
@@ -247,7 +266,7 @@ class CommandBuilder:
     """
 
     @classmethod
-    def coerce(cls, obj, atomic_default=False) -> 'CommandBuilder':
+    def coerce(cls, obj, atomic_default=False) -> "CommandBuilder":
         """
         Create a :class:`CommandBuilder` from the objects that an event handler is allowed to
         return.
@@ -269,13 +288,15 @@ class CommandBuilder:
         self._commands = [[]]  # type: List[List[Command]]
         self._defaults = CommandDefaults()
 
-    def defaults(self,
-                 party: Optional[Party] = None,
-                 ledger_id: Optional[str] = None,
-                 workflow_id: Optional[str] = None,
-                 application_id: Optional[str] = None,
-                 command_id: Optional[str] = None,
-                 deduplication_time: Optional[timedelta] = None) -> None:
+    def defaults(
+        self,
+        party: Optional[Party] = None,
+        ledger_id: Optional[str] = None,
+        workflow_id: Optional[str] = None,
+        application_id: Optional[str] = None,
+        command_id: Optional[str] = None,
+        deduplication_time: Optional[timedelta] = None,
+    ) -> None:
         if party is not None:
             self._defaults.default_party = party
         if ledger_id is not None:
@@ -289,18 +310,20 @@ class CommandBuilder:
         if deduplication_time is not None:
             self._defaults.default_deduplication_time = deduplication_time
 
-    def create(self, template, arguments=None) -> 'CommandBuilder':
+    def create(self, template, arguments=None) -> "CommandBuilder":
         return self.append(create(template, arguments=arguments))
 
-    def exercise(self, contract, choice, arguments=None) -> 'CommandBuilder':
+    def exercise(self, contract, choice, arguments=None) -> "CommandBuilder":
         return self.append(exercise(contract, choice, arguments=arguments))
 
-    def create_and_exercise(self, template, create_arguments, choice_name, choice_arguments=None) \
-            -> 'CommandBuilder':
-        return self.append(create_and_exercise(
-            template, create_arguments, choice_name, choice_arguments))
+    def create_and_exercise(
+        self, template, create_arguments, choice_name, choice_arguments=None
+    ) -> "CommandBuilder":
+        return self.append(
+            create_and_exercise(template, create_arguments, choice_name, choice_arguments)
+        )
 
-    def append(self, *commands: CommandsOrCommandSequence) -> 'CommandBuilder':
+    def append(self, *commands: CommandsOrCommandSequence) -> "CommandBuilder":
         """
         Append one or more commands, or list of commands to the :class:`CommandBuilder` in flight.
         This method respects the value of ``atomic_default`` that this object was constructed with.
@@ -318,43 +341,51 @@ class CommandBuilder:
         else:
             return self.append_nonatomically(*commands)
 
-    def append_atomically(self, *commands: Union[Command, Sequence[Command]]) -> 'CommandBuilder':
+    def append_atomically(self, *commands: Union[Command, Sequence[Command]]) -> "CommandBuilder":
         self._commands.extend([flatten_command_sequence(commands)])
         return self
 
-    def append_nonatomically(self, *commands: Union[Command, Sequence[Command]]) -> \
-            'CommandBuilder':
+    def append_nonatomically(
+        self, *commands: Union[Command, Sequence[Command]]
+    ) -> "CommandBuilder":
         self._commands.extend([[cmd] for cmd in flatten_command_sequence(commands)])
         return self
 
-    def build(self, defaults: 'Optional[CommandDefaults]' = None) -> 'Collection[CommandPayload]':
+    def build(self, defaults: "Optional[CommandDefaults]" = None) -> "Collection[CommandPayload]":
         """
         Return a collection of commands.
         """
         if defaults is None:
-            raise ValueError('defaults must currently be specified')
+            raise ValueError("defaults must currently be specified")
 
-        command_id = defaults.default_command_id or self._defaults.default_command_id or \
-                     uuid.uuid4().hex
+        command_id = (
+            defaults.default_command_id or self._defaults.default_command_id or uuid.uuid4().hex
+        )
 
-        return [CommandPayload(
-            party=defaults.default_party or self._defaults.default_party,
-            ledger_id=defaults.default_ledger_id or self._defaults.default_ledger_id,
-            workflow_id=defaults.default_workflow_id or self._defaults.default_workflow_id,
-            application_id=defaults.default_application_id or self._defaults.default_application_id,
-            command_id=command_id,
-            deduplication_time=defaults.default_deduplication_time or self._defaults.default_deduplication_time,
-            commands=commands
-        ) for i, commands in enumerate(self._commands) if commands]
+        return [
+            CommandPayload(
+                party=defaults.default_party or self._defaults.default_party,
+                ledger_id=defaults.default_ledger_id or self._defaults.default_ledger_id,
+                workflow_id=defaults.default_workflow_id or self._defaults.default_workflow_id,
+                application_id=defaults.default_application_id
+                or self._defaults.default_application_id,
+                command_id=command_id,
+                deduplication_time=defaults.default_deduplication_time
+                or self._defaults.default_deduplication_time,
+                commands=commands,
+            )
+            for i, commands in enumerate(self._commands)
+            if commands
+        ]
 
     def __format__(self, format_spec):
-        if format_spec == 'c':
+        if format_spec == "c":
             return str(self._commands)
         else:
             return repr(self)
 
     def __repr__(self):
-        return f'CommandBuilder({self._commands})'
+        return f"CommandBuilder({self._commands})"
 
 
 def flatten_command_sequence(commands: Sequence[CommandsOrCommandSequence]) -> List[Command]:
@@ -381,8 +412,10 @@ def flatten_command_sequence(commands: Sequence[CommandsOrCommandSequence]) -> L
                     else:
                         errors.append(((i, j), cmd))
     if errors:
-        raise ValueError(f'Failed to interpret some elements as Commands in the list: '
-                         f'$[{index}] = {command}' for index, command in errors)
+        raise ValueError(
+            f"Failed to interpret some elements as Commands in the list: " f"$[{index}] = {command}"
+            for index, command in errors
+        )
     return ret
 
 
@@ -422,34 +455,41 @@ class CommandPayload:
         will be deduplicated. Duplicate commands submitted before the end of this window return an
         ``ALREADY_EXISTS`` error.
     """
+
     party: Party
     ledger_id: str
     workflow_id: str
     application_id: str
     command_id: str
-    commands: 'Sequence[Command]'
-    deduplication_time: 'Optional[timedelta]' = None
+    commands: "Sequence[Command]"
+    deduplication_time: "Optional[timedelta]" = None
 
     def __post_init__(self):
-        missing_fields = [field.name for field in fields(self)
-                          if field.name != 'deduplication_time' and getattr(self, field.name) is None]
+        missing_fields = [
+            field.name
+            for field in fields(self)
+            if field.name != "deduplication_time" and getattr(self, field.name) is None
+        ]
         if missing_fields:
-            raise ValueError(f'Some fields are set to None when they are required: '
-                             f'{missing_fields}')
+            raise ValueError(
+                f"Some fields are set to None when they are required: " f"{missing_fields}"
+            )
 
 
 def create(template, arguments=None):
     if not isinstance(template, str):
         raise ValueError(
-            'template must be a string name, a template type, or an instantiated template')
+            "template must be a string name, a template type, or an instantiated template"
+        )
 
     return CreateCommand(template, arguments)
 
 
 def exercise(contract, choice, arguments=None):
     if not isinstance(choice, str):
-        raise ValueError('choice must be a string name, a template type, '
-                         'or an instantiated template')
+        raise ValueError(
+            "choice must be a string name, a template type, " "or an instantiated template"
+        )
 
     return ExerciseCommand(contract, choice, arguments)
 
@@ -467,11 +507,11 @@ class Serializer:
     Serializer interface for objects on the write-side of the API.
     """
 
-    def serialize_value(self, tt: 'Type', obj: 'Any') -> 'Any':
-        raise NotImplementedError('serialize_value requires an implementation')
+    def serialize_value(self, tt: "Type", obj: "Any") -> "Any":
+        raise NotImplementedError("serialize_value requires an implementation")
 
-    def serialize_command(self, command: 'Any') -> 'Any':
-        raise NotImplementedError('serialize_command requires an implementation')
+    def serialize_command(self, command: "Any") -> "Any":
+        raise NotImplementedError("serialize_command requires an implementation")
 
 
 class AbstractSerializer(Serializer):
@@ -480,21 +520,22 @@ class AbstractSerializer(Serializer):
     serialization have been implemented.
     """
 
-    def __init__(self, lookup: 'SymbolLookup'):
+    def __init__(self, lookup: "SymbolLookup"):
         self.lookup = lookup
 
     @property
-    def mapper(self) -> 'ValueMapper':
-        raise NotImplementedError(f'{type(self)}.mapper() must be defined')
+    def mapper(self) -> "ValueMapper":
+        raise NotImplementedError(f"{type(self)}.mapper() must be defined")
 
-    def serialize_value(self, tt: 'Type', obj: Any) -> 'Any':
+    def serialize_value(self, tt: "Type", obj: Any) -> "Any":
         from ..values import Context
+
         return Context(self.mapper, self.lookup).convert(tt, obj)
 
-    def serialize_commands(self, commands: 'Sequence[Command]') -> 'Sequence[Any]':
+    def serialize_commands(self, commands: "Sequence[Command]") -> "Sequence[Any]":
         return [self.serialize_command(cmd) for cmd in commands]
 
-    def serialize_command(self, command: 'Command') -> 'Any':
+    def serialize_command(self, command: "Command") -> "Any":
         if isinstance(command, CreateCommand):
             name = self.lookup.template_name(command.template_type)
             value = self.serialize_value(con(name), command.arguments)
@@ -513,7 +554,8 @@ class AbstractSerializer(Serializer):
             choice = find_choice(template, command.choice)
             choice_args = self.serialize_value(choice.arg_binder.type, command.choice_argument)
             return self.serialize_create_and_exercise_command(
-                name, create_value, choice.name, choice_args)
+                name, create_value, choice.name, choice_args
+            )
 
         elif isinstance(command, ExerciseByKeyCommand):
             name = self.lookup.template_name(command.template_type)
@@ -521,27 +563,27 @@ class AbstractSerializer(Serializer):
             key_value = self.serialize_value(template.key.type, command.contract_key)
             choice = find_choice(template, command.choice)
             choice_args = self.serialize_value(choice.arg_binder.type, command.choice_argument)
-            return self.serialize_exercise_by_key_command(
-                name, key_value, choice.name, choice_args)
+            return self.serialize_exercise_by_key_command(name, key_value, choice.name, choice_args)
 
         else:
-            raise ValueError(f'unknown Command type: {command!r}')
+            raise ValueError(f"unknown Command type: {command!r}")
 
-    def serialize_create_command(self, name: 'TypeConName', template_args: 'Any') -> 'Any':
-        raise NotImplementedError('serialize_create_command requires an implementation')
+    def serialize_create_command(self, name: "TypeConName", template_args: "Any") -> "Any":
+        raise NotImplementedError("serialize_create_command requires an implementation")
 
     def serialize_exercise_command(
-            self, contract_id: 'ContractId', choice_name: str, choice_args: 'Any') -> 'Any':
-        raise NotImplementedError('serialize_exercise_command requires an implementation')
+        self, contract_id: "ContractId", choice_name: str, choice_args: "Any"
+    ) -> "Any":
+        raise NotImplementedError("serialize_exercise_command requires an implementation")
 
     def serialize_exercise_by_key_command(
-            self, name: 'TypeConName', key_arguments: 'Any',
-            choice_name: str, choice_arguments: 'Any') -> 'Any':
-        raise NotImplementedError(
-            'serialize_exercise_by_key_command requires an implementation')
+        self, name: "TypeConName", key_arguments: "Any", choice_name: str, choice_arguments: "Any"
+    ) -> "Any":
+        raise NotImplementedError("serialize_exercise_by_key_command requires an implementation")
 
     def serialize_create_and_exercise_command(
-            self, name: 'TypeConName', create_args: 'Any',
-            choice_name: str, choice_arguments: 'Any') -> 'Any':
+        self, name: "TypeConName", create_args: "Any", choice_name: str, choice_arguments: "Any"
+    ) -> "Any":
         raise NotImplementedError(
-            'serialize_create_and_exercise_command requires an implementation')
+            "serialize_create_and_exercise_command requires an implementation"
+        )
