@@ -13,26 +13,36 @@ def unmangle_name(s: str) -> str:
     :param s: The string to interpret.
     :return: A new string with the appropriate substitutions.
     """
-    return s.replace('$u002b', '+').replace('$u005b', '[').replace('$u005d', ']'). \
-        replace('$u003c', '<').replace('$u003e', '>').replace('$u003a', ':'). \
-        replace('$u0022', '"').replace('$u0028', '(').replace('$u0029', ')'). \
-        replace('$u002f', '/').replace('$u002c', ',').replace('$u003d', '=')
+    return (
+        s.replace("$u002b", "+")
+        .replace("$u005b", "[")
+        .replace("$u005d", "]")
+        .replace("$u003c", "<")
+        .replace("$u003e", ">")
+        .replace("$u003a", ":")
+        .replace("$u0022", '"')
+        .replace("$u0028", "(")
+        .replace("$u0029", ")")
+        .replace("$u002f", "/")
+        .replace("$u002c", ",")
+        .replace("$u003d", "=")
+    )
 
 
-def is_hidden_module_name(name: 'Sequence[str]') -> bool:
-    if len(name) == 1 and name[0] in ('GhcPrim', 'Prelude'):
+def is_hidden_module_name(name: "Sequence[str]") -> bool:
+    if len(name) == 1 and name[0] in ("GhcPrim", "Prelude"):
         return True
     if len(name) >= 2:
-        if name[0] == 'GHC':
+        if name[0] == "GHC":
             return True
-        elif name[0] == 'DA':
+        elif name[0] == "DA":
             return True
-        elif name[0] == 'Control' and name[1] == 'Exception':
+        elif name[0] == "Control" and name[1] == "Exception":
             return True
     return False
 
 
-def maybe_parentheses(obj, operator: str = ' ') -> str:
+def maybe_parentheses(obj, operator: str = " ") -> str:
     """
     Wrap the string (or object's string representation) in parentheses, but only if required.
     :param obj:
@@ -49,23 +59,23 @@ def maybe_parentheses(obj, operator: str = ' ') -> str:
     # if there are already wrapping punctuation marks, there may not be a need to add additional
     # ones
     open_mark = s[0]
-    if open_mark in '([{':
+    if open_mark in "([{":
         groupings = [s[0]]
         for c in s[1:-1]:
-            if c in '([{':
+            if c in "([{":
                 groupings.append(c)
-            elif c == '}':
-                if groupings[-1] == '{':
+            elif c == "}":
+                if groupings[-1] == "{":
                     groupings.pop()
                 else:
                     groupings.clear()
-            elif c == ']':
-                if groupings[-1] == '[':
+            elif c == "]":
+                if groupings[-1] == "[":
                     groupings.pop()
                 else:
                     groupings.clear()
-            elif c == ')':
-                if groupings[-1] == '(':
+            elif c == ")":
+                if groupings[-1] == "(":
                     groupings.pop()
                 else:
                     groupings.clear()
@@ -73,12 +83,14 @@ def maybe_parentheses(obj, operator: str = ' ') -> str:
             if not groupings:
                 # we balanced out all groupings (or just his a syntax error in unmatched groupings);
                 # add clarifying parentheses
-                return '(' + s + ')'
-        if (groupings[0] == '(' and groupings[-1] == ')') or \
-           (groupings[0] == '[' and groupings[-1] == ']') or \
-           (groupings[0] == '{' and groupings[-1] == '}'):
+                return "(" + s + ")"
+        if (
+            (groupings[0] == "(" and groupings[-1] == ")")
+            or (groupings[0] == "[" and groupings[-1] == "]")
+            or (groupings[0] == "{" and groupings[-1] == "}")
+        ):
             return s
-    return '(' + s + ')' if operator in s else s
+    return "(" + s + ")" if operator in s else s
 
 
 def indent(text: str, spaces: int):
@@ -86,5 +98,5 @@ def indent(text: str, spaces: int):
     Prepend every line of the specified text with a set number of spaces. Line endings are
     preserved.
     """
-    prefix = ' ' * spaces
-    return ''.join(prefix + t for t in text.splitlines(keepends=True))
+    prefix = " " * spaces
+    return "".join(prefix + t for t in text.splitlines(keepends=True))
