@@ -6,7 +6,7 @@ Endpoints for managing parties/bots connected via a dazl client.
 """
 
 from dataclasses import asdict, dataclass
-from typing import Collection, Sequence, TYPE_CHECKING
+from typing import TYPE_CHECKING, Collection, Sequence
 
 from ..client import Bot, _NetworkImpl
 from ..model.core import DazlImportError, Party, SourceLocation
@@ -17,7 +17,7 @@ if TYPE_CHECKING:
 
 @dataclass(frozen=True)
 class BotInfoList:
-    bots: 'Collection[BotInfo]'
+    bots: "Collection[BotInfo]"
 
 
 @dataclass(frozen=True)
@@ -26,60 +26,64 @@ class BotInfo:
     name: str
     party: Party
     state: str
-    entries: 'Sequence[BotInfoEntry]'
+    entries: "Sequence[BotInfoEntry]"
 
 
 @dataclass(frozen=True)
 class BotInfoEntry:
     event_key: str
-    source_location: 'SourceLocation'
+    source_location: "SourceLocation"
 
 
-def build_routes(network_impl: '_NetworkImpl') -> 'Collection[web.AbstractRouteDef]':
+def build_routes(network_impl: "_NetworkImpl") -> "Collection[web.AbstractRouteDef]":
     try:
         from aiohttp import web
     except ImportError:
-        raise DazlImportError('aiohttp', 'server routes could not be built because aiohttp is not installed')
+        raise DazlImportError(
+            "aiohttp", "server routes could not be built because aiohttp is not installed"
+        )
 
     routes = web.RouteTableDef()
 
-    @routes.get('/parties')
-    async def get_parties(_: 'web.Request') -> 'web.Response':
+    @routes.get("/parties")
+    async def get_parties(_: "web.Request") -> "web.Response":
         json_dict = dict(parties=[{"id": party} for party in network_impl.parties()])
         return web.json_response(json_dict)
 
-    @routes.get('/bots')
-    async def get_bots(_: 'web.Request') -> 'web.Response':
-        bot_infos = [_get_bot_info(bot)
-                     for party_impl in network_impl.party_impls()
-                     for bot in party_impl.bots]
+    @routes.get("/bots")
+    async def get_bots(_: "web.Request") -> "web.Response":
+        bot_infos = [
+            _get_bot_info(bot)
+            for party_impl in network_impl.party_impls()
+            for bot in party_impl.bots
+        ]
         return web.json_response(asdict(BotInfoList(bots=bot_infos)))
 
-    @routes.get('/bots/{bot_id}')
-    async def get_bot_by_id(request: 'web.Request') -> 'web.Response':
+    @routes.get("/bots/{bot_id}")
+    async def get_bot_by_id(request: "web.Request") -> "web.Response":
         bot = get_bot(request)
         return web.json_response(asdict(_get_bot_info(bot)))
 
-    @routes.post('/bots/{bot_id}/state/{action}')
-    async def change_bot_state(request: 'web.Request') -> 'web.Response':
+    @routes.post("/bots/{bot_id}/state/{action}")
+    async def change_bot_state(request: "web.Request") -> "web.Response":
         bot = get_bot(request)
-        action = request.match_info['action']
+        action = request.match_info["action"]
 
-        if action.lower() == 'pause':
+        if action.lower() == "pause":
             bot.pause()
             raise web.HTTPAccepted()
-        elif action.lower() == 'resume':
+        elif action.lower() == "resume":
             bot.resume()
             raise web.HTTPAccepted()
         else:
             raise web.HTTPBadRequest()
 
-    def get_bot(request: 'web.Request') -> 'Bot':
+    def get_bot(request: "web.Request") -> "Bot":
         """
         Return the bot referenced by the request. Throw HTTPNotFound if a bot with the ID specified
         in the URL does not exist.
         """
-        bot_id = request.match_info['bot_id']
+        bot_id = request.match_info["bot_id"]
         bot = network_impl.find_bot(bot_id)
         if bot is None:
             raise web.HTTPNotFound()
@@ -88,9 +92,12 @@ def build_routes(network_impl: '_NetworkImpl') -> 'Collection[web.AbstractRouteD
     return routes
 
 
-def _get_bot_info(bot: 'Bot') -> 'BotInfo':
-    entries = [BotInfoEntry(event_key=entry.event_key, source_location=entry.source_location)
-               for entry in bot.entries()]
+def _get_bot_info(bot: "Bot") -> "BotInfo":
+    entries = [
+        BotInfoEntry(event_key=entry.event_key, source_location=entry.source_location)
+        for entry in bot.entries()
+    ]
 
     return BotInfo(
-        id=bot.id, party=bot.party, name=bot.name, state=bot.state.value, entries=entries)
+        id=bot.id, party=bot.party, name=bot.name, state=bot.state.value, entries=entries
+    )
