@@ -1,12 +1,12 @@
 # Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
 # SPDX-License-Identifier: Apache-2.0
 
-from pathlib import Path
 from os import PathLike
+from pathlib import Path
 from typing import Union
 
 
-def pathify(path: 'Union[str, Path, PathLike]') -> Path:
+def pathify(path: "Union[str, Path, PathLike]") -> Path:
     """
     Convert an object that could be a :class:`Path` into a :class:`Path`.
     """
@@ -15,4 +15,4 @@ def pathify(path: 'Union[str, Path, PathLike]') -> Path:
     elif isinstance(path, PathLike):
         return Path(path)
     else:
-        raise ValueError(f'path must be a str or Path (got {path!r} instead)')
+        raise ValueError(f"path must be a str or Path (got {path!r} instead)")
