@@ -5,18 +5,18 @@
 This module contains helper methods for detecting terminal capabilities.
 """
 
-from subprocess import Popen, DEVNULL, PIPE
+from subprocess import DEVNULL, PIPE, Popen
 from typing import Optional, Tuple
 
 
-def termsize() -> 'Tuple[Optional[int], Optional[int]]':
+def termsize() -> "Tuple[Optional[int], Optional[int]]":
     """
     Return the current size of the terminal. If the current terminal is not a tty, then
     ``(None, None)`` is returned.
     """
     try:
-        with Popen(['stty', 'size'], stdout=PIPE, stderr=DEVNULL) as proc:
-            term_size_str = proc.stdout.read().decode('utf8') if proc.stdout is not None else None
+        with Popen(["stty", "size"], stdout=PIPE, stderr=DEVNULL) as proc:
+            term_size_str = proc.stdout.read().decode("utf8") if proc.stdout is not None else None
     except FileNotFoundError:
         term_size_str = None
     if term_size_str is not None:
@@ -35,11 +35,11 @@ def print_termcap() -> None:
     """
     rows, cols = termsize()
     if rows is not None and cols is not None:
-        print('The terminal size is:')
+        print("The terminal size is:")
         print(termsize())
     else:
-        print('Not a terminal.')
+        print("Not a terminal.")
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     print_termcap()
